#!/usr/bin/env python3

import socket
import base64
import os
import re
import signal
import time
import datetime
import hereiam_config
import hereiam_log

terminate = False

DEBUG = False
VERSION = "0.0.3"
OS = os.uname().sysname


#given two strings, being the username and password
#encode for passing to the Zoneedit dns update
def authority(uname, pwd):
	uandp = uname + ":" + pwd
	auth = base64.b64encode(bytes(uandp, 'utf-8'))

	return auth.decode('utf-8')


# This function is a wrapper for the socket.send function
def output(sock, buf):
 
	newbuf = bytes(buf, 'utf-8')
	
	#output to the socket
	if sock.fileno() > 0:
		sock.sendall(newbuf)
		return 0
	else:
		return -1
			
			
def read_input(sock):
	bufsize = 4096
	#output to the socket
	if sock.fileno() > 0:
		return sock.recv(bufsize)
	else:
		return bytes("\0")


def do_connect(server, port):
	# try to create a socket object
	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	if sock == None:
		return None
		
	# create an addres for the connect function
	address = (server, port)	

	# try to connect
	try:
		sock.connect(address)
	except (socket.timeout):
		sock.close()
		return None
		
	return sock

			
#server get request
def server_get_request(config):

	sock = do_connect(config['server'], config['port'])
	if sock == None:
		return
	
	authorisation = authority(config['username'], config['authentication'])
	crlf = "\r\n"
	
	buffer = "GET " + config['url'] + "?action=edit&"
	output(sock, buffer)
	
	if config['address'] != "":
		buffer = "myip=" + config['address'] + "&"
		output(sock, buffer)
	
	buffer = "partner=" + config['partner'] + "&"
	output(sock, buffer)
	
	if config['wildcard'] == "yes":
		buffer = "wildcard=ON&"
	else:
		buffer = "wildcard=OFF&"
	
	output(sock, buffer)
	
	buffer = "hostname=" + config['hostname'] + "&"
	output(sock, buffer)
	
	buffer = " HTTP/1.0" + crlf
	output(sock, buffer)
	
	buffer = "Authorization: Basic " + authorisation + crlf
	output(sock, buffer)
	
	buffer = "User-Agent: " + config['partner'] + "-" + VERSION + " " + OS + "(by Ray Anderson)" + crlf
	output(sock, buffer)
	
	buffer = "Host: " + config['server'] + crlf
	output(sock, buffer)
	
	output(sock, crlf)

	inbuffer = read_input(sock)
	
	sock.close()
	
	strmsg = inbuffer.decode('utf-8')
	msgbodystart = strmsg.find("\r\n\r\n",0) + 4
	
	matchobj = re.search(r"<.*>", strmsg)
	
	if matchobj == None:
		return "No response from server"
		
	return "Response: " + matchobj.group()[1:len(matchobj.group())-1]


# this is a GET request to the hostname
# an OK response will indicate that the
# DNS entry does not need updating
def get_testbytes(config):
	crlf = "\r\n"
	hostname = config['hostname']
	hostip = socket.gethostbyname(hostname)
	host = socket.gethostbyaddr(hostip)[0]
	url = config['testurl']
	agent = "hereiam"
	checkport= config['testport']
	
	testbytes = bytes("hereiam", 'utf-8')
	
	message = "GET " + url + " HTTP/1.1" + crlf
	message += "HOST: " + host + crlf
	message += "User-Agent: " + agent + crlf
	message += crlf
	
	c_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	c_sock.settimeout(20)
	try:
		c_sock.connect((hostip, checkport))
	except socket.timeout:
		c_sock.close()
		return 1		
	except OSError as error:
		c_sock.close()
		return 2
	
	c_sock.sendall(bytes(message,'utf-8'))
	response = c_sock.recv(4096)
	c_sock.close()
	
	retval_OK = "HTTP/1.1 200 OK"
	strResponse = response.decode('utf-8')
	
	if (strResponse.find(retval_OK, 0, len(retval_OK)) == 0):
		return 0
	else:
		return 3


# check the ip address is correctly pointing to the hereiam site
def check_ip(configuration):
	timeout = 5
	while (timeout > 0):
		retval = get_testbytes(configuration)
		
		if retval == 0:
			# the website was found using the hostname, so the DNS has the correct IP Address
			return 0
		elif retval == 1:
			# there was a timeout. Try again, but decrement the timeout
			timeout -= 1
			if terminate == True:
				return 1
		elif retval == 2:
			# the website was not found using the hostname, so the DNS has the correct IP Address
			return 2
		elif retval == 3:
			# the website was found but there was an error with the page
			return 3
		else:
			# Not sure how we get here, log. In daemon mode, this will check again once the delay time has expired.
			return 4
		
	
	
	return 1
		
# custom handler for signals to elegantly shutdown the app.
def my_custom_handler(signum, stack_frame):
	global terminate
	terminate = True

# main loop	
def main():
	global terminate
	
	configpath = "/volume1/Web/hereiam/config/hereiam.config"
	logfilepath = "/volume1/Web/hereiam/config/hereiam.log"
	hereiam_log.log(logfilepath, 'Starting up')
	
	configuration = hereiam_config.update_config(configpath, logfilepath, None)
	
	previousupdatetime = datetime.datetime.now() - datetime.timedelta(seconds=(configuration['delay'] * 2))
		
	while(True):
		#I will be in this loop forever, until I break thanks to a termination signal.
		
		# check to see if the config file has been updated while the the app has been running.
		configuration = hereiam_config.update_config(configpath, logfilepath, configuration)
		
		if configuration['enabled'] == "Yes" or \
			configuration['enabled'] == "yes" or \
			configuration['enabled'] == "y" or \
			configuration['enabled'] == "Y":

			if datetime.datetime.now() > (previousupdatetime + datetime.timedelta(seconds=configuration['delay'])):
				if configuration['autocheck'] == "yes" or \
					configuration['autocheck'] == "Yes" or \
					configuration['autocheck'] == "y" or \
					configuration['autocheck'] == "Y":
					
					# can now check
					retval = check_ip(configuration)
					if retval == 0 or retval == 3:
						# appears to be up to date
						# nothing to do
						if retval == 0:
							hereiam_log.log(logfilepath, 'IP address correctly set.')
						else:
							hereiam_log.log(logfilepath, "Error[3]: testurl page error")
					else:
						# some other error, just try and update
						# update the website
						#send the GET request to the server once
						hereiam_log.log(logfilepath, "Autocheck could not contact " + configuration['hostname'])
						if retval == 1:
							hereiam_log.log(logfilepath, "Error[1]: timeout when checking DNS IP")
						elif retval == 2:
							hereiam_log.log(logfilepath, "Error[2]: DNS IP not correct. Update required")
						else:
							hereiam_log.log(logfilepath, "Error[4]: unknown")
						
						hereiam_log.log(logfilepath, "Update GET request")
						response = server_get_request(configuration)			
						hereiam_log.log(logfilepath, response)
				else:
					hereiam_log.log(logfilepath, "Update GET request")
					response = server_get_request(configuration)			
					hereiam_log.log(logfilepath, response)
				
				previousupdatetime = datetime.datetime.now()
				
			if terminate == True:
				hereiam_log.log(logfilepath, 'Termination signal received')
				break
		
		time.sleep(1)
		
	hereiam_log.log(logfilepath, 'Shutting down')


if __name__ == '__main__':
	signal.signal(signal.SIGTERM, my_custom_handler)
	signal.signal(signal.SIGINT, my_custom_handler)

	main()
	